import json

from django import forms
from django.forms.utils import flatatt
from django.utils.encoding import force_text
from django.utils.html import format_html
from django.utils.safestring import mark_safe

INTL_TEL_INPUT_VERSION = '15.0.1'


class IntlTelInputWidget(forms.TextInput):
    input_type = 'tel'

    class Media:
        css = {
            'all': (
                'https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/'
                '{version}/css/intlTelInput.css'.format(
                    version=INTL_TEL_INPUT_VERSION
                ),
            ),
        }
        js = (
            'https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/'
            '{version}/js/intlTelInput-jquery.min.js'.format(
                version=INTL_TEL_INPUT_VERSION
            ),
        )

    def __init__(self, attrs=None, allow_dropdown=True,
                 preferred_countries=['us', 'gb'], default_code='us',
                 use_default_init=True):

        if use_default_init:
            self.Media.js += ('intl_tel_input/init.js',)

        final_attrs = {
            'size': '20',
            'data-allow-dropdown': allow_dropdown,
            'data-preferred-countries': json.dumps(preferred_countries),
            'data-default-code': default_code,
        }

        if attrs is not None:
            final_attrs.update(attrs)

        super(IntlTelInputWidget, self).__init__(attrs=final_attrs)

    def render(self, name, value, renderer=None, attrs=None):
        if value is None:
            value = ''

        final_attrs = self.build_attrs(attrs, self.attrs)
        final_attrs['data-hidden-name'] = name

        if value != '':
            final_attrs['value'] = force_text(self.format_value(value))

        final_attrs['class'] = ' '.join([
            'intl-tel-input', final_attrs.get('class', '')
        ]).strip()

        output = format_html('<input{}>', flatatt(final_attrs))
        return mark_safe(output)



from django.conf.urls import url

from .views import attrs_test, home, initial_test, two_fields_test

urlpatterns = [
    url(r'^$', home),
    url(r'^attrs-test/$', attrs_test),
    url(r'^initial-test/$', initial_test),
    url(r'^two-field-test/$', two_fields_test),
]



from django.http import HttpResponseRedirect
from django.shortcuts import render

from .forms import TelForm, TelFormAttrs, TwoTelForm


def home(request):
    if request.POST:
        form = TelForm(request.POST)
        return HttpResponseRedirect('{path}?ok'.format(path=request.path))
    else:
        form = TelForm()

    return render(request, 'home.html', {'form': form})


def attrs_test(request):
    form = TelFormAttrs()
    return render(request, 'home.html', {'form': form})


def initial_test(request):
    form = TelForm(initial={'tel_number': '+81123456789'})
    return render(request, 'home.html', {'form': form})


def two_fields_test(request):
    if request.POST:
        form = TwoTelForm(request.POST)
    else:
        form = TwoTelForm()
    return render(request, 'home.html', {'form': form})



from __future__ import unicode_literals

import html5lib
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.test import TestCase

from selenium.webdriver import Firefox
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait


class IntlTelInputTest(TestCase):
    def assertValidHTML(self, content, msg=None):
        parser = html5lib.HTMLParser()
        parser.parse(content)
        if parser.errors:
            default_msg = ['Content is invalid HTML:']
            lines = content.split('\n')
            for position, errorcode, datavars in parser.errors:
                default_msg.append(
                    '  %s' % html5lib.constants.E[errorcode] % datavars
                    )
                default_msg.append('    %s' % lines[position[0] - 1])

            msg = self._formatMessage(msg, '\n'.join(default_msg))
            raise self.failureException(msg)

    def test_render(self):
        r = self.client.get('/')
        self.assertValidHTML(r.content)
        self.assertEqual(r.status_code, 200)

    def test_static(self):
        r = self.client.get('/')
        self.assertIn(
            '/js/intlTelInput-jquery.min.js',
            r.content.decode('utf-8')
        )
        self.assertIn('/intl_tel_input/init.js', r.content.decode('utf-8'))
        self.assertIn('/css/intlTelInput.css', r.content.decode('utf-8'))

    def test_post(self):
        r = self.client.post('/', {'tel_number': '+81123456789'})
        self.assertRedirects(r, '/?ok')

    def test_with_attrs(self):
        r = self.client.get('/attrs-test/')
        self.assertIn('title="Telephone number"', r.content.decode('utf-8'))
        self.assertIn('data-default-code="jp"', r.content.decode('utf-8'))
        self.assertIn('data-preferred-countries="[&quot;jp&quot;]"',
                      r.content.decode('utf-8'))
        self.assertIn('placeholder="foobar"', r.content.decode('utf-8'))

    def test_with_initial(self):
        r = self.client.get('/initial-test/')
        self.assertIn('value="+81123456789"', r.content.decode('utf-8'))


class wait_for_utils_script(object):
    def __call__(self, driver):
        return driver.execute_script(
            "return window.intlTelInputUtils !== undefined"
        )


class AcceptanceTest(StaticLiveServerTestCase):
    def setUp(self):
        options = Options()
        options.headless = True
        self.driver = Firefox(options=options)

    def tearDown(self):
        self.driver.quit()

    def test_two_inputs(self):
        driver = self.driver
        driver.get('{live_server_url}/two-field-test/'.format(
            live_server_url=self.live_server_url
        ))
        WebDriverWait(driver, 5).until(
            wait_for_utils_script()
        )
        inputs = driver.find_elements_by_css_selector('input.intl-tel-input')
        inputs[0].send_keys('555-5555')
        inputs[1].send_keys('555-4444')
        inputs[1].send_keys(Keys.RETURN)
        WebDriverWait(driver, 5).until(
            EC.presence_of_element_located((By.ID, 'success-text'))
        )

        self.assertIn('Form is valid', driver.page_source)
